/*-
 * Copyright (C)2007 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2007 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: pgsadump.c,v 1.5 2007/03/07 22:47:46 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sysexits.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <err.h>
#include <babolo/BLINflag.h>
#include <mife.h>
#include "pgsadump.h"

static void
usage(int ex) {
    int i;

    fprintf( stderr,
             "pgsadump " VERS "  " DATE " @BABOLO http://www.babolo.ru/\n"
             "pgsadump [-012345678qSVxXv.. ][-b blcksz ][-i INDEX_MAX_KEYS ]file [[_]type..]\n"
             "types are:\n"
           );
    for (i = 0; PGsaDumpCollection[i].conv; i++) fprintf(stderr, "  %s\n", PGsaDumpCollection[i].descr);
    exit(ex);
}

int
main(int argc, char **argv, char **envp) {
    struct mife_descriptor *in = NULL;
    int ex = EX_OK, ch, i, j;
    u_int32_t pgn;
    off_t pos;
    ssize_t last;
    PGsaDumpCtl ctl;

    ctl.flags = BLIN_VER1;
    ctl.blksz = 8192;
    ctl.nbase = 10000;
    ctl.ind_maxk = 32;
    ctl.atrmax = 1664;
    ctl.atrmin = 0;
    ctl.atrterm = 0;
    ctl.atrmask = 0;
    ctl.alignmax = sizeof(int);
    while ((ch = getopt(argc, argv, "0123456789a:A:b:cehi:lLm:M:N:qStvVxX:")) != EOF)
        switch (ch) {
         case '0': ctl.flags |= PGsaDumpNULS;               break;
         case '1': ctl.flags |= PGsaDumpCTID;               break;
         case '2': ctl.flags |= PGsaDumpOID;                break;
         case '3': ctl.flags |= PGsaDumpXMIN;               break;
         case '4': ctl.flags |= PGsaDumpCMIN;               break;
         case '5': ctl.flags |= PGsaDumpXMAX;               break;
         case '6': ctl.flags |= PGsaDumpCMAX;               break;
         case '7': ctl.flags |= PGsaDumpINFO;               break;
         case '8': ctl.flags |= PGsaDumpNATR;               break;
         case '9': ctl.flags |= PGsaDumpRTID;               break;
         case 'a': ctl.atrmin = strtoul(optarg, NULL, 0);   break;
         case 'A': ctl.atrmax = strtoul(optarg, NULL, 0);   break;
         case 'b': ctl.blksz = strtoul(optarg, NULL, 0);    break;
         case 'c': ctl.flags |= PGsaDumpXCOL;               break;
         case 'e': ctl.flags |= PGsaDumpETID;               break;
         case 'h': usage(ex);                               break;
         case 'i': ctl.ind_maxk = strtoul(optarg, NULL, 0); break;
         case 'l': ctl.flags |= PGsaDumpLNUS;               break;
         case 'L': ctl.flags |= PGsaDumpLDEL;               break;
         case 'm': ctl.atrterm = strtoul(optarg, NULL, 0);  break;
         case 'M': ctl.atrmask = strtoul(optarg, NULL, 0);  break;
         case 'N': ctl.nbase = strtoul(optarg, NULL, 0);    break;
         case 'q': setBLIN_VERMASK(ctl.flags, 0);           break;
         case 'S': ctl.flags |= PGsaDumpXBSZ;               break;
         case 't': ctl.flags |= PGsaDumpXTUP;               break;
         case 'v': BLIN_VERBOSE(ctl.flags);                 break;
         case 'V': ctl.flags |= PGsaDumpXVER;               break;
         case 'x': ctl.flags |= PGsaDumpXATR;               break;
         case 'X': ctl.alignmax = strtoul(optarg, NULL, 0); break;
         default:  usage(EX_USAGE);
        }
    argc -= optind;
    argv += optind;
    if  (argc > 0 && !argv[0]) usage(EX_USAGE);
    if  (!(in = mife_open(kukMIFE_BUFL(1), argv[0]))) {
        ifBLIN_QV1(ctl.flags) warn("file %s", argv[0]);
        exit(EX_NOINPUT);
    }
    ctl.atrs[0] = argc - 1;
    for (i = 1; i < argc && argv[i]; i++) {
        ch = 1;
        if  (argv[i][0] == '_') {
            for (j = 0; PGsaDumpCollection[j].conv; j++) {
                if  (!strcmp(&(argv[i][1]), PGsaDumpCollection[j].descr)) {
                    ctl.atrs[i] = j | 0x8000;
                    ch = 0;
                    break;
            }   }
        } else {
            for (j = 0; PGsaDumpCollection[j].conv; j++) {
                if  (!strcmp(argv[i], PGsaDumpCollection[j].descr)) {
                    ctl.atrs[i] = j;
                    ch = 0;
                    break;
        }   }   }
        if  (ch) {
            ifBLIN_QV1(ctl.flags) fprintf(stderr, "Illegal type: %s\n", argv[i]);
            exit(EX_DATAERR);
    }   }
    ifBLIN_QV3(ctl.flags) {
        for (i = 0; i <= ctl.atrs[0]; i++) {
            fprintf(stderr, "  [%d]=%d\n", i, ctl.atrs[i]);
    }   }
    for (pos = 0, pgn = 0; (last = mife_read(in, 0, pos, NULL)) >= ctl.blksz; pos += ctl.blksz, pgn++) {
        ifBLIN_QV3(ctl.flags) fprintf(stderr, "pageN=%d, offset=%lld\n", pgn, pos);
        if  ((ex = PGsaDumpBlock(&ctl, pgn, mife_get(in, pos)))) break;
    }
    if  (last < 0) {
        ifBLIN_QV2(ctl.flags) warn("input");
        exit(EX_DATAERR);
    }
    if  (last != 0) {
        ifBLIN_QV2(ctl.flags) warnx("truncated");
        exit(EX_DATAERR);
    }
    mife_close(in);
    exit(ex);
}
